;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x779ed36e49a2436b8082e523e5dec303 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xb7e7c7468b319cee3f4fbe803210f101 . "./code.cg2js/cg-printing.fasl")
(#xd7eb642cac1691b4b3181396958c08fb . "./code.cg2js/cg-caret.fasl")
(#x222e58b53a4e12bb31e6cdf9e1595992 . "./code.cg2js/cg-item-list.fasl")
(#x32afbf1d606cfc546735b4076198d4e5 . "./code.cg2js/cg-progress-indicator.fasl")
(#x73429495df5b890bf02a8ac764f0d79e . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xeffd752b8e3c61498e1bc75244b6e73a . "./code.cg2js/cg-outline.fasl")
(#x99137d635134c4b0c81f906ad4c6f2be . "./code.cg2js/cg-palette.fasl")
(#x48e93713d21a7acaa21a0b32805c8129 . "./code.cg2js/cg-selected-object.fasl")
(#x874a21558aa709ea282b2fac05c39ddb . "./code.cg2js/cg-status-bar.fasl")
(#xd42b59fb400b4db678ff58e0bd04e87d . "./code.cg2js/cg-wrap-string.fasl")
(#x46045d49624859c4f8cb37adfbb9bb1b . "./code.cg2js/cg-fill-texture.fasl")
(#xa5ab9e22347c6d15dc8597b3864b653c . "./code.cg2js/cg-layout.fasl")
(#xfeb35f0e95ed2261c0c31c2f28d96286 . "./code.cg2js/cg-timer.fasl")
(#xb7abe298848bf1fcc5a1ce25161e7462 . "./code.cg2js/cg-dialog-item.fasl")
(#xb41067b39cf2a000882091bfda446907 . "./code.cg2js/cg-button.fasl")
(#x1106ef04456fa55a12dfa08f677426b9 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xb737a2db7504214d074d19207e7315ee . "./code.cg2js/cg-tray.fasl")
(#x83204ce026579cd726e58be781ea9573 . "./code.cg2js/cg-plot-widget.fasl")
(#xc2424f6a72c27e059ff1ca956d807983 . "./code.cg2js/cg-pixmap.fasl")
(#xdfd065d962bd5f59cfb573bbe16529ab . "./code.cg2js/cg-font-dialog.fasl")
(#xc8b632656d5d9dacc1c8bf795776a6b2 . "./code.cg2js/cg-os-widget.fasl")
(#x129cb7d5f9f57d2d4f5df4aba3efffdd . "./code.cg2js/cg-text-or-combo.fasl")
(#x379d49b943450b1b1818d459a0e906cf . "./code.cg2js/cg-trackbar.fasl")
(#x40affd15682b0ff35b5584e523a90e27 . "./code.cg2js/cg-up-down-control.fasl")
(#xd3fc883b065671fd7d71fdf9cbf5c14e . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x979856912b20f3838250fc969341ba43 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xd22b61a73ed9610544b2b5bca6f368e1 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x0de9134f578224f3b235bf275f41f35d . "./code.cg2js/cg-bitmap-stream.fasl")
(#x17655af3a5eff8137a50dd36ca76172f . "./code.cg2js/cg-chart-widget.fasl")
(#x9bb89e55a4ef1c8a7779d4978995d0fd . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x73c9e8c1ec1bc9075af5b20b3be417f6 . "./code.cg2js/cg-group-box.fasl")
(#xf6a07f84016e74634ee37d23735a0e17 . "./code.cg2js/cg-static-text.fasl")
(#xe3029b739f14788af1420b7922fd2823 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x8e459abd12871f9f0be7c27c0cd931a7 . "./code.cg2js/cg-color-dialog.fasl")
(#x9e02715460bb1f4ea1952a512db38a91 . "./code.cg2js/cg-intersect.fasl")
(#x4453067b1f8a6c9b052508f568ac6719 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x910922de641f75fd4acf00b71cafa549 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x7c62f5127feada8c5961925334a06cec . "./code.cg2js/cg-html-dialog.fasl")
(#xa946848d1cde1b2b41bdbccb164d094e . "./code.cg2js/cg-lettered-menu.fasl")
(#x33f4a3759951b4646d33482eb42cc160 . "./code.cg2js/cg-class-grid.fasl")
(#x42ecb93428005e958beb5fb9a919830d . "./code.cg2js/cg-pixmap-printing.fasl")
(#xec4a40a49b570cba02e6f01d240a3668 . "./code.cg2js/cg-calendar.fasl")
(#xfab79efb535605c2c3e27229e13d37d5 . "./code.cg2js/cg-editable-text.fasl")
(#x8384a33736aef2b8a832730ec55a52b3 . "./code.cg2js/cg-property.fasl")
(#xdff6c64bd175e3c81cf6904d3409fa65 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xafb886b2f0baa5955f5674c241ad1373 . "./code.cg2js/cg-grid-widget.fasl")
(#xad64880d4550bbd8f1d72b1013df9c6f . "./code.cg2js/cg-paren-matching.fasl")
(#xa712547d1d9bf8b6001c64564f2dd95d . "./code.cg2js/cg-dev.fasl")
(#x8ac9e78f08da296d51917f8589b85ddc . "./code.cg2js/cg-drawable.fasl")
(#x85a8adae0f8b4dbbc8225b7ff1008c85 . "./code.cg2js/cg-curve.fasl")
(#x9456d7875020a720c0a649f73c4e7b81 . "./code.cg2js/cg-lisp-widget.fasl")
(#x0e39d3c22e4b2506c0963d47e6b2b6ee . "./code.cg2js/cg-dde-utils.fasl")
(#xc5d951fda3c41bafb70e940605179ba5 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xced83b8ace2bee62953fb66942b52d1c . "./code.cg2js/cg-object-editor-layout.fasl")
(#x0001ac1fbb3e3a11f66935d1ca8d3332 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xf6a942ca9849a9232236435f65d2da76 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xdeb9b12c915a2cb994f8eb1549b43864 . "./code.cg2js/cg-lamp.fasl")
(#xffc523a83dc8758e72c15209982f0d77 . "./code.cg2js/cg-common-control.fasl")
(#xdb674b8eed47311d35144515a6067ab5 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x3837ae78035d2dcaa86e174e5aee5089 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xe166ad51c1c5f84bbb659ff977b3a475 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x34834d127b1520c583d41d759e3a2d62 . "./code.cg2js/cg-project-window.fasl")
(#x7926e9ef1214d645e7f7542f173a8b36 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x5e6f1b5048486b86a81fb4a2a101a338 . "./code.cg2js/cg-class-support.fasl")
(#x1fd37c6a08d974d6d6199460906bf99e . "./code.cg2js/cg-multi-picture-button.fasl")
(#x7b839b39970f3119262424f23379a97c . "./code.cg2js/cg-yes-no-list.fasl")
(#x4bdb044d71bd8d1dd4b5654d7ef74d90 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x4cad2c85c62cdac6ddc6002fb85ce3f6 . "./code.cg2js/cg-string-dialog.fasl")
(#x26e938a4728fa3b81fa8ee4d45d88bd2 . "./code.cg2js/cg-menu-tooltip.fasl")
(#xf26aa1914af2484362a0522d6051b7a9 . "./code.cg2js/cg-dropping-outline.fasl")
(#x749aec5323fd3086cd011202e6642506 . "./code.cg2js/cg.fasl")
(#x3f152c3cbf47bc5f9586ab7b7bbb4a73 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xbb766f5d74ed75b36fee06808eb9bb9d . "./code.cg2js/cg-scaling-stream.fasl")
(#x55c3a2fae23f690aa601295ea08460af . "./code.cg2js/cg-yes-no-string.fasl")
(#x83e5860d5cf6bc8b30f267b36acce6b4 . "./code.cg2js/cg-icon-pixmap.fasl")
(#xe9c6e4523e3871ced65eca7e24a47616 . "./code.cg2js/cg-web-browser.fasl")
(#x885e30e5fc72a41d6c9c5cb151989a03 . "./code.cg2js/cg-clipboard-stack.fasl")
(#xf400d7f4401fd233c9dacc0b37fa6186 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x6270d84325580a608b584d90b528e891 . "./code.cg2js/cg-message-dialog.fasl")
(#xc501eb1d9bdbebd6c9ce34301200b941 . "./code.cg2js/cg-comtab.fasl")
(#x4a5a81a359332b6f02238b6e681e5bb1 . "./code.cg2js/cg-base.fasl")
(#x020c221a8d80a0622c5a5ad246be52dc . "./code.cg2js/cg-pixmap-widget.fasl")
(#xded3910df8f8afddd7393b3c186b1b56 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x86e186b85d6c4b0c11c41f203ce6f579 . "./code.cg2js/cg-text-edit-pane.fasl")
(#xe6430db6ab4fe22f91eeb8e58ff17293 . "./code.cg2js/cg-utility-dialog.fasl")
(#xa93ea1b952b18c3c86a2f324c8b9e337 . "./code.cg2js/cg-object-editor.fasl")
(#x0d7a55ada2098b3efc3c2eb26e69a9d3 . "./code.cg2js/cg-edit-in-place.fasl")
(#x0b1ab5fcaaa7242b5120b25bb2a0a21d . "./code.cg2js/cg-radio-button.fasl")
(#x31de8708a7bbfa58024da2113fc55c5a . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x023179e1fe881630249e3f4bf87b1983 . "./code.cg2js/cg-nodes.fasl")
(#xe91728b3f50a0d3acd005fd82ec55abf . "./code.cg2js/cg-get-position.fasl")
(#xd01e90c74cb0f8033236bf518cd2249a . "./code.cg2js/cg-directory-dialog.fasl")
(#x07a40a7f22b548d5009c516088037d83 . "./code.cg2js/cg-lisp-text.fasl")
(#x60174cfdccbae95becc85773dcad625d . "./code.cg2js/cg-combo-box.fasl")
(#x49c602e0d09cb7e27c75dbc25095b4b3 . "./code.cg2js/cg-text-widget.fasl")
(#x9e1184d6ee1bbcd0f369df8707c97f59 . "./code.cg2js/cg-list-view.fasl")
(#x90f0916a3a2f9f916f5587b66a8e4497 . "./code.cg2js/cg-header-control.fasl")
(#x460697eeb782d664f0721fc70ba4ac08 . "./code.cg2js/cg-graphics-context.fasl")
(#xbc8e638354c5a790d93f852087dc1ce8 . "./code.cg2js/cg-get-pixmap.fasl")
(#x208667d2d364811ecbcb779096d6f716 . "./code.cg2js/cg-menu.fasl")
(#x1cb4f8fb3758adbba206c6c244670658 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x4aa5294e281ff71f03d0a36801f1748a . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x2f46ee14cf8c4589fbac24d58353a40b . "./code.cg2js/cg-tooltip.fasl")
(#x2dc7654b66f021f9dd82ad2a767e70aa . "./code.cg2js/cg-html-widget.fasl")
(#xe049557cee0021fa1eebd443c37face0 . "./code.cg2js/cg-choice-list.fasl")
(#x4a85d3ce07c8e4911fa3be603310b67c . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#xc588eb3eed3c28fd4030e14679659ad4 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x700a63195feb366cf78af6f895396d3f . "./code.cg2js/cg-icon.fasl")
(#x0087a40077f284247600ece39c6d99fb . "./code.cg2js/cg-acache.fasl")
(#x84292ecefe964f7f1980603ce0309372 . "./code.cg2js/cg-template-string.fasl")
(#x9b9fdbaafc169c51bd7a60b11c423bc2 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x68c7c353a82ef4b2063d34742a13f84c . "./code.cg2js/cg-toggling-widget.fasl")
(#xa1cb7af67dd78c9addbd3fa46281f45d . "./code.cg2js/cg-tab-control.fasl")
(#x6e997b81f28ad5b1908c7aa2f28c7b17 . "./code.cg2js/cg-split-bar.fasl")
(#xeb28ef948749d95d9d787052ab085e19 . "./code.cg2js/cg-picture-widget.fasl")
(#xf68bb00b6801d99aba9c8e0447dd90c1 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xdf34e5a9b685817edbdda1634f800ff8 . "./code.cg2js/cg-choose-printer.fasl")
(#x4f7364c9eba55538915801178bbb9be1 . "./code.cg2js/cg-os-window.fasl")
(#x5a6a7ed9915defaba0523a341c42f253 . "./code.cg2js/cg-scroll-bar.fasl")
(#x29f67baf8be9b946029a2e6ea67b68b0 . "./code.cg2js/cg-clipboard.fasl")
(#x98fe07031cf3798ae22cfeaae60f9e6f . "./code.cg2js/cg-find-string-dialog.fasl")
(#xbe762eb819f61494d57374b83e662316 . "./code.cg2js/cg-hotspot.fasl")
(#xa51950688930619e74c10db5d2b4f069 . "./code.cg2js/cg-rich-edit.fasl")
(#x101ae2460e8f125bd14f8b1963431f75 . "./code.cg2js/cg-check-box.fasl")
(#x88a12650d123142be545fd826bf1794f . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x8c053ea5d21c82f7b9244a9b60616d70 . "./code.cg2js/cg-file-dialog.fasl")
(#xe252033670a4e9f035db68c91f8a80b1 . "./code.cg2js/cg-toolbar.fasl")
(#xc4b17995df8151b84ced623150bf5803 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xca081ec2b7981836e0f1fe4804af8087 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x2ee112c69c95cb309843dec594d01702 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x11ac96f1325323929aa49976004469a4 . "./code.idejs/ide-project.fasl")
(#xf613b3083bfa73ab519fd30d03db2e0c . "./code.idejs/ide-intro.fasl")
(#xc131698d4df81ac50d8dc5810a3b0f0b . "./code.idejs/ide-find-in-files.fasl")
(#x4ed0a0a8ddc4415b09ea624165fb42f1 . "./code.idejs/ide-grapher.fasl")
(#xecd535c5e5d1d93569b206bfecbf2126 . "./code.idejs/ide-inspector.fasl")
(#x93ad184d96d4df484955ac1c0c3891b2 . "./code.idejs/ide-editor.fasl")
(#x82e5cb77b6d76aa32f0236c414971c92 . "./code.idejs/ide-window-list.fasl")
(#x08c7a3b4990f37ff5309445988f6fb00 . "./code.idejs/ide-process-browser.fasl")
(#x6de617abc563923cb5c074268b05772b . "./code.idejs/ide-patch-dialog.fasl")
(#x1dcb15ffdf2f84f5917b05472fd7fc29 . "./code.idejs/ide-class-grapher.fasl")
(#x528ab48ec1883b22a18aacba76b7d636 . "./code.idejs/ide-base.fasl")
(#xdcda5c40d5fb5ec2ad8860a5195b9da8 . "./code.idejs/ide.fasl")
(#x7495a87727717770906d4322da9184a3 . "./code.idejs/ide-trace.fasl")
(#xc5f28386810887f7cbea6ccaccee4b10 . "./code.idejs/ide-form.fasl")
(#x90c24c53a932849c51524c76c1ab6bfe . "./code.idejs/ide-find-definitions.fasl")
(#x26f554197cee99032739c7a838223e5c . "./code.idejs/ide-apropos.fasl")
(#x6dbbeb0722e961831e328bfd5f530c7b . "./code.idejs/ide-class-browser.fasl")
(#x79af575e3dcfd54f5a0987888d2293f3 . "./code.idejs/ide-navigator.fasl")
(#xce0b56bd4b12644fdf6a206105ded266 . "./code.idejs/ide-clipboard.fasl")
(#x6cb3671fb0ddf2b9990f319072dc7d36 . "./code.idejs/ide-acache.fasl")
(#x24eb605e26237014b828c260841bfc57 . "./code.idejs/ide-keyshort.fasl")
(#xa151bbd57b38e7e2edb60e554a98f9d4 . "./code.idejs/ide-package-list.fasl")
(#xb576af0168874592ecfd3252d94e5333 . "./code.idejs/ide-dev.fasl")
(#xb19a66ec109be5a69fb5293c577fc104 . "./cgjs.html")
(#x34779e5d4bbb71d45bf63f88fac54100 . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
